import java.awt.AWTEvent;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

public class SeparatorTree
	 extends JTree 
	 implements ActionListener, TreeSelectionListener{

	 private DefaultTreeModel model = null;
	 private JButton button = null;

	 static public int VALUES = 27;
	 static public int INT_VALUES[] = {'#','A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P','Q','R','S','T','U','V','W','X','Y','Z'};
	 static public String STRING_VALUES[] = {"#","A","B","C","D","E","F","G","H","I","J","K","L","M","N","O","P","Q","R","S","T","U","V","W","X","Y","Z"};
	 
	 public SeparatorTree() {
		  super(new DefaultTreeModel(new DefaultMutableTreeNode("Root")));
		  addTreeSelectionListener(this);
		  // Build initial tree.
		  this.getSelectionModel().setSelectionMode(TreeSelectionModel.CONTIGUOUS_TREE_SELECTION);
		  model = (DefaultTreeModel) getModel();
		  DefaultMutableTreeNode root = (DefaultMutableTreeNode) model.getRoot();
		  for(int i = 0; i < VALUES; i++) {
				model.insertNodeInto(new SeparatorNode(STRING_VALUES[i]), root, i);
		  }
		  this.expandPath(new TreePath(root));
		  this.setRootVisible(false);
	 }

	 public void actionPerformed(ActionEvent event) {
		  if(getSelectionCount() == 1) {
				System.err.println("Split current node");
		  }
		  else {
				TreePath paths[] = getSelectionPaths();
				// Arg. Look through paths, and see if the end nodes are merged, and if so unmerge them and build a new TreePath with only valid unmerged paths.
				DefaultMutableTreeNode parent = (DefaultMutableTreeNode)paths[0].getPathComponent(paths[0].getPathCount() - 2); // Second to last. At worst 'Root'.
				SeparatorNode first = (SeparatorNode)paths[0].getLastPathComponent();
				SeparatorNode last = (SeparatorNode)paths[paths.length - 1].getLastPathComponent();
				int index = model.getIndexOfChild(parent, first);
				for(int i = 0; i < paths.length; i++) {
					 SeparatorNode node = (SeparatorNode)paths[i].getLastPathComponent();
					 System.err.println("Removing " + node);
					 model.removeNodeFromParent(node);
				}
				model.insertNodeInto(new SeparatorNode(first.toString(), last.toString()), parent, index);
		  }
	 }

	 public JButton getButton() {
		  if(button == null) {
				button = new JButton("Select Rows");
				button.addActionListener(this);
				button.setEnabled(false);
		  }
		  return button;
	 }

	 public void valueChanged(TreeSelectionEvent event) {
		  if(getSelectionCount() == 1) {
				TreePath path = getSelectionPath();
				SeparatorNode node = (SeparatorNode) path.getLastPathComponent();
				if(node.isMerged()) {
					 button.setText("Restore");
					 button.setEnabled(true);
				}
				else {
					 button.setText("Select Rows");
					 button.setEnabled(false);
				}
		  }
		  else if(getSelectionCount() >= 2) {
				button.setText("Merge");
				button.setEnabled(true);
		  }
		  else {
				button.setText("Select Rows");
				button.setEnabled(false);
		  }
	 }

	 private class SeparatorNode
		  extends DefaultMutableTreeNode {
		  private boolean mapped = false;
		  private String start = null;
		  private String end = null;
		  private Vector records = null;
		  public SeparatorNode() {
		  }
		  public SeparatorNode(String s) {
				start = s;
		  }
		  public SeparatorNode(String s, String e) {
				start = s;
				end = e;
		  }
		  public void addRecord(String name) {
				records.add(name);
		  }
		  public TreeNode getChildAt(int index) {
				System.err.println("Called getChildAt(" + index + ")");
				if(!mapped) {
					 mapped = true;
					 if(end == null) {
						  for(int i = 1; i < VALUES; i++) {
								add(new SeparatorNode(start + STRING_VALUES[i]));
						  }
					 }
					 else {
						  for(String prefix = start; prefix != null && prefix.compareTo(end) <= 0; prefix = increment(prefix)) {
								for(int i = 1; i < VALUES; i++) {
									 add(new SeparatorNode(prefix + STRING_VALUES[i]));
								}
						  }
					 }
				}
				return super.getChildAt(index);
		  }
		  public int getChildCount() {
				int count = 0;
				if(mapped) {
					 count = super.getChildCount();
				}
				else if(!start.equals(STRING_VALUES[0])) {
					 if(end == null) {
						  count = VALUES - 1;
					 }
					 else {
						  System.err.println("Calculating children from merged node.");
						  for(String prefix = start; prefix != null && prefix.compareTo(end) <= 0; prefix = increment(prefix)) {
								count = count + VALUES - 1;
						  }
					 }
				}
				return count;
		  }		
		  public boolean isMerged() {
				if(end != null) {
					 return true;
				}
				return false;
		  }
		  public String toString() {
				if(end == null) {
					 return start;
				}
				else {
					 return start + " - " + end;
				}
		  }
		  public String increment(String str) {
				System.err.print("Incrementing " + str + " -> ");
				char chars[] = str.toCharArray();
				boolean found = false;
				int pos = str.length() - 1;
				while(found == false && pos >= 0) {
					 if((int)chars[pos] + 1 <= INT_VALUES[26]) {
						  chars[pos] = (char)((int)chars[pos] + 1);
						  found = true;
					 }  
					 else {
						  chars[pos] = (char)INT_VALUES[1];
						  pos--;
					 }
				}
				// If we haven't found it yet, return null.
				if(!found) {
					 return null;
				}
				System.err.println(String.valueOf(chars));
				return String.valueOf(chars);
		  }
	 }

	 static public void main(String args[]) {
		  SeparatorTree tree = new SeparatorTree();
		  JFrame frame = new JFrame();
		  frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		  frame.setSize(new Dimension(400,400));
		  JPanel pane = (JPanel)frame.getContentPane();
		  pane.setLayout(new BorderLayout());
		  pane.add(new JScrollPane(tree), BorderLayout.CENTER);
		  pane.add(tree.getButton(), BorderLayout.SOUTH);
		  frame.show(); 
	 }
}
